//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_VEHICLE_H
#define PROTOCOL_VEHICLE_H

#include <thread>
#include "VehicleEndpoint.h"
#include "PlatoonService.h"
#include <NetworkingLib/Networking.h>
#include <NetworkingLib/ServiceServer.h>
#include <NetworkingLib/ServiceClient.h>

namespace protocol
{
class Vehicle : public std::enable_shared_from_this<Vehicle>
{
public:
    using Ptr = std::shared_ptr<Vehicle>;

    enum class Role
    {
        LEADER, FOLLOWER
    };

    virtual ~Vehicle()
    {}

    bool isIdle();

    bool isPlatoonRunning();

    void createPlatoon();

    void leavePlatoon();

    virtual void stop();

    Role getRole() const noexcept
    { return role; }

protected:
    Vehicle(
        networking::Networking & net,
        VehicleId vehicleId,
        Role role);

    enum class State
    {
        IDLE,
        CREATING_PLATOON,
        RUNNING_PLATOON
    };

    PlatoonId platoonId;
    std::atomic<PlatoonConfig> platoonConfig;
    State state{State::IDLE};
    std::mutex stateMutex;
    networking::Networking & net;
    networking::service::Server<PlatoonService>::Ptr platoonCreateServiceServer;
    networking::service::Client<PlatoonService>::Ptr platoonCreateServiceClient;
    std::vector<VehicleEndpoint> availableVehicleEndpoints;
    std::vector<VehicleEndpoint>::iterator vehicleEndpointIter;

    virtual void doCreatePlatoon();

    virtual void doRunPlatoon() = 0;

    virtual void doLeavePlatoon() = 0;

    virtual void makePlatoonCreateRequest(const VehicleEndpoint & vehicleEndpoint) = 0;

    virtual PlatoonMessage makePlatoonCreateResponse(
        const tcp::endpoint & clientEndpoint, const PlatoonMessage & request) = 0;

    void makePlatoonCreateRequestToNextEndpoint();

    void scanAvailableVehicles();

    void setState(State newState);

    VehicleEndpoint getOwnEndpoint() const;

private:
    Role role;
    VehicleId vehicleId;
};
}


#endif //PROTOCOL_VEHICLE_H
