//
// Created by philipp on 02.12.17.
//

#include <iostream>
#include "ProtocolTest.h"
#include "../include/LeaderVehicle.h"
#include "../include/FollowerVehicle.h"

using namespace protocol;

void test::testBasicSetup()
{
    networking::Networking net;

    auto leader = protocol::LeaderVehicle::create(net, 1);
    auto follower1 = protocol::FollowerVehicle::create(net, 2);
    auto follower2 = protocol::FollowerVehicle::create(net, 3);

    leader->setPlatoonConfig(protocol::PlatoonConfig{10.0f, 20.0f});

    int configCount1 = 0;
    follower1->setOnUpdatePlatoonConfigCallback(
        [&configCount1](const auto & config)
        {
            std::cout << "Follower[2] received config[" << configCount1 << "]: {ps: " << config.platoonSpeed << ", ipd: "
                      << config.innerPlatoonDistance << "}\n";
            configCount1++;
        });

    int configCount2 = 0;
    follower2->setOnUpdatePlatoonConfigCallback(
        [&configCount2](const auto & config)
        {
            std::cout << "Follower[3] received config[" << configCount2 << "]: {ps: " << config.platoonSpeed << ", ipd: "
                      << config.innerPlatoonDistance << "}\n";
            configCount2++;
        });

    leader->createPlatoon();
    follower1->createPlatoon();
    follower2->createPlatoon();

    sleep(1);

    leader->setPlatoonConfig(protocol::PlatoonConfig{5.0f, 10.0f});

    sleep(3);

    std::cout << "Follower[2] will now leave the platoon\n";

    follower1->leavePlatoon();

    sleep(1);

    std::cout << "Follower[3] will now leave the platoon\n";

    follower2->leavePlatoon();

    sleep(1);

    std::cout << "Leader[1] will now leave the platoon\n";

    leader->leavePlatoon();

    sleep(1);
}

void test::testStop()
{
    networking::Networking net;

    // Test if the destruction works as expected.
    {
        auto leader = LeaderVehicle::create(net, 1);
        leader->createPlatoon();
        sleep(1);
    }
    std::cout << "LEADER SUCCESS!\n";
    sleep(1);
    {
        auto follower = FollowerVehicle::create(net, 1);
        follower->createPlatoon();
        sleep(1);
    }
    std::cout << "FOLLOWER SUCCESS!\n";
}
